/**************************************************************************
    Souliss - 
    Gateway for Boulleville
    * Arduino Mega
        - on board LED on D13
    * Ethernet shield (W5100)
        - D10 : SS
    * TEP custom Shield
        - LEDs (Green 34, RED 35, Pink 36 & 37)
        - WS2811 output on D7 & D8 for RJ45 1 & 2
        - 1Wire on D6 & D14 for RJ45 1 & 2
        - RS485-2 : D5=RX/TX, TX2=D16, RX2=D17, Serial2
        - RS485-1 : D2=RX/TX, TX1=D18, RX1=D19, Serial1
        - RFM12B :  D3=irq, PB0=D53:SS
                    PB3/D50=MISO, PB2/D51=MOSI, PB1/D52=CLK

        - Baro : D20D/D21 I2C
        - D11 : PWM buzzer
        - D42-D49= input_opto_x
        - D22-D29 : relays
        - D30-D33 : unused 
        - D38-41 : unused
        - D12 / D13 : unused
        - D4 unused
        - D15/RX3 unused
        - ADC0-ADC7 unused
    * RS485
        A/B lines are on RJ45 1-2
***************************************************************************/

// USART Config
#define USARTBAUDRATE_INSKETCH
#define USART_BAUD19k2 1

// RS485-2
#define USARTDRIVER_INSKETCH
#define USART_TXENABLE			1
#define USART_TXENPIN			5
#define	USARTDRIVER	            Serial2

// Ethernet options -------------------------------------------------------
#define MAC_INSKETCH
uint8_t MAC_ADDRESS[] = {0x90, 0xA2, 0xDA, 0x0E, 0xC0, 0x55};
#define AUTO_MAC    0

// Souliss options --------------------------------------------------------
// Configure the framework
#include "bconf/StandardArduino.h"          // Use a standard Arduino
#include "conf/ethW5100.h"                  // Ethernet through Wiznet W5100
#include "conf/usart.h"    
#include "conf/Gateway.h"                   // The main node is the Gateway

// Include framework code and libraries
#include <SPI.h>

/*** All configuration includes should be above this line ***/ 
#include "Souliss.h"

enum {
    LOGIC_TEP_VMC_ETAGE,    // 2 slots
    logic_tep_spare1,
    LOGIC_TEP_ATM_PRESSURE, // 2 slots
    logic_tep_spare2,
    LOGIC_TEP_BUZZER,
    LOGIC_TEP_LED_ROSE_1,
    LOGIC_TEP_NB
};

// Define the network configuration according to your router settings
uint8_t ip_address[4]  = {192, 168, 1, 70};
uint8_t subnet_mask[4] = {255, 255, 255, 0};
uint8_t ip_gateway[4]  = {192, 168, 1, 1};
#define myvNet_address      ip_address[3]       // The last byte of the IP address is also the vNet address

// Define the RS485 network configuration
#define myvNet_subnet   0xFF00
#define Gateway_RS485   0xCE01
#define Peer_RS485      0xCE02

//============================================================================
void setup()
{
    Serial.begin(115200);
    Initialize();

    // Set Ethernet network parameters
    Souliss_SetIPAddress(ip_address, subnet_mask, ip_gateway);                          // Address on the Ethernet interface
    SetAsGateway(myvNet_address);                                                       // Set this node as gateway for SoulissApp  

    // Set RS485 parameters
    SetAddress(Gateway_RS485, myvNet_subnet, 0);                    // Set the address on the RS485 bus

    // This node as gateway will get data from the Peer
    SetAsPeerNode(Peer_RS485, 1); 
    
    Set_T19(LOGIC_TEP_VMC_ETAGE);               // 2 slots
    Set_Pressure(LOGIC_TEP_ATM_PRESSURE);       // T53  /!\ 2 slots!
    Set_PulseOutput(LOGIC_TEP_BUZZER);          // T14  / 1 slot
    Set_SimpleLight(LOGIC_TEP_LED_ROSE_1);      // Define a simple LED light logic
}

// Mean value of atm pressure = 1013.25 hPa
#define DEADBAND_PRESSURE       0.0001      // Deadband value 0.01%, 0.10hPa for 1023°

//============================================================================
void loop()
{ 
    // Here we start to play
    EXECUTEFAST() {                     
        UPDATEFAST();   

        FAST_50ms() {   // We process the logic and relevant input and output every 50 milliseconds
            // TYPICALS ------------------------------------------------------
            Logic_SimpleLight(LOGIC_TEP_LED_ROSE_1);
            Logic_PulseOutput(LOGIC_TEP_BUZZER);
            Souliss_Logic_T58(memory_map, LOGIC_TEP_ATM_PRESSURE, DEADBAND_PRESSURE, &data_changed);
        } 

        FAST_1110ms() {
            static float pressure = 1000.0;
            pressure += 1.0;
            Souliss_ImportAnalog(memory_map, LOGIC_TEP_ATM_PRESSURE, &pressure);
        }
        // This node does just networking, bridging the Peer node to the Ethernet network
        FAST_GatewayComms();
    }
} 
